package ru.dbotthepony.mc.otm.block;

import com.google.common.collect.ImmutableMap;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.EntityBlock;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockBehaviour;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.material.Material;
import net.minecraft.world.level.material.MaterialColor;
import net.minecraft.world.phys.shapes.BooleanOp;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.phys.shapes.VoxelShape;
import ru.dbotthepony.mc.otm.capability.IMatterGridCell;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityMatterCable;

import javax.annotation.Nullable;
import java.util.ArrayList;

public class BlockMatterCable extends Block implements EntityBlock {
	@Nullable
	@Override
	public BlockEntity newBlockEntity(BlockPos blockPos, BlockState blockState) {
		return new BlockEntityMatterCable(blockPos, blockState);
	}

	public static final BooleanProperty CONNECTION_SOUTH = BooleanProperty.create("connect_south");
	public static final BooleanProperty CONNECTION_WEST = BooleanProperty.create("connect_west");
	public static final BooleanProperty CONNECTION_EAST = BooleanProperty.create("connect_east");
	public static final BooleanProperty CONNECTION_NORTH = BooleanProperty.create("connect_north");
	public static final BooleanProperty CONNECTION_UP = BooleanProperty.create("connect_up");
	public static final BooleanProperty CONNECTION_DOWN = BooleanProperty.create("connect_down");

	public static final BooleanProperty[] MAPPING_CONNECTION_PROP = new BooleanProperty[] {
		CONNECTION_DOWN,
		CONNECTION_UP,
		CONNECTION_NORTH,
		CONNECTION_SOUTH,
		CONNECTION_WEST,
		CONNECTION_EAST
	};

	protected ImmutableMap<BlockState, VoxelShape> SHAPES;

	public BlockMatterCable() {
		super(BlockBehaviour.Properties.of(Material.STONE, MaterialColor.METAL).requiresCorrectToolForDrops().strength(1.0F, 6.0F));
		registerDefaultState(
			this.defaultBlockState()
				.setValue(CONNECTION_DOWN, false)
				.setValue(CONNECTION_UP, false)
				.setValue(CONNECTION_NORTH, false)
				.setValue(CONNECTION_SOUTH, false)
				.setValue(CONNECTION_WEST, false)
				.setValue(CONNECTION_EAST, false)
		);

		SHAPES = getShapeForEachState((state) -> {
			ArrayList<VoxelShape> shapes = new ArrayList<>();

			if (state.getValue(CONNECTION_SOUTH)) {
				shapes.add(Shapes.box(
					0.5 - 0.15,
					0.5 - 0.15,
					0.65,

					0.5 + 0.15,
					0.5 + 0.15,
					1
				));
			}

			if (state.getValue(CONNECTION_NORTH)) {
				shapes.add(Shapes.box(
					0.5 - 0.15,
					0.5 - 0.15,
					0,

					0.5 + 0.15,
					0.5 + 0.15,
					0.35
				));
			}

			if (state.getValue(CONNECTION_DOWN)) {
				shapes.add(Shapes.box(
					0.5 - 0.15,
					0,
					0.5 - 0.15,

					0.5 + 0.15,
					0.5 - 0.15,
					0.5 + 0.15
				));
			}

			if (state.getValue(CONNECTION_UP)) {
				shapes.add(Shapes.box(
					0.5 - 0.15,
					0.5 - 0.15,
					0.5 - 0.15,

					0.5 + 0.15,
					1,
					0.5 + 0.15
				));
			}

			if (state.getValue(CONNECTION_EAST)) {
				shapes.add(Shapes.box(
					0.65,
					0.5 - 0.15,
					0.5 - 0.15,

					1,
					0.5 + 0.15,
					0.5 + 0.15
				));
			}

			if (state.getValue(CONNECTION_WEST)) {
				shapes.add(Shapes.box(
					0,
					0.5 - 0.15,
					0.5 - 0.15,

					0.35,
					0.5 + 0.15,
					0.5 + 0.15
				));
			}

			VoxelShape final_shape = CORE_SHAPE;

			for (VoxelShape add_shape : shapes) {
				final_shape = Shapes.joinUnoptimized(final_shape, add_shape, BooleanOp.OR);
			}

			return final_shape;
		});
	}

	private final VoxelShape CORE_SHAPE = Shapes.box(
		0.5 - 0.15,
		0.5 - 0.15,
		0.5 - 0.15,

		0.5 + 0.15,
		0.5 + 0.15,
		0.5 + 0.15
	);

	@Override
	public VoxelShape getShape(BlockState p_60555_, BlockGetter p_60556_, BlockPos p_60557_, CollisionContext p_60558_) {
		VoxelShape get = SHAPES.get(p_60555_);
		return get != null ? get : CORE_SHAPE;
	}

	@Override
	public boolean hasDynamicShape() {
		return true;
	}

	// doesn't work at all
	/*
	@Nullable
	@Override
	public BlockState getStateForPlacement(BlockPlaceContext p_49820_) {
		Level level = p_49820_.getLevel();

		if (level.isClientSide)
			return this.defaultBlockState();

		BlockState state = this.defaultBlockState();

		for (int i = 0; i < MAPPING_CONNECTION_PROP.length; i++) {
			OverdriveThatMatters.LOGGER.info("{} {} {}", Direction.values()[i], p_49820_.getClickedPos().offset(Direction.values()[i].getNormal()), level.getBlockEntity(p_49820_.getClickedPos().offset(p_49820_.getClickedFace().getNormal()).offset(Direction.values()[i].getNormal())));

			if (
				level.getBlockEntity(p_49820_.getClickedPos().offset(Direction.values()[i].getNormal())) instanceof IMatterGridCell cell && cell.isValidMatterCell()// ||
				//level.getBlockState(p_49820_.getClickedPos().offset(p_49820_.getClickedFace().getNormal()).offset(Direction.values()[i].getNormal())).getBlock() instanceof BlockMatterCable
			) {
				state.setValue(MAPPING_CONNECTION_PROP[i], true);
			}
		}

		return state;
	}
	 */

	// blocks
	@Override
	public void neighborChanged(BlockState self, Level level, BlockPos position_self, Block block_notifier, BlockPos position_notifier, boolean unknown_flag) {
		Direction normal = Direction.fromNormal(position_notifier.subtract(position_self));
		boolean updated = false;
		BlockEntity get_entity = level.getBlockEntity(position_notifier);

		if (!self.getValue(MAPPING_CONNECTION_PROP[normal.ordinal()]) && get_entity instanceof IMatterGridCell cell && cell.isValidMatterCell()) {
			self = self.setValue(MAPPING_CONNECTION_PROP[normal.ordinal()], true);
			updated = true;
		} else if (self.getValue(MAPPING_CONNECTION_PROP[normal.ordinal()]) && (get_entity == null || get_entity instanceof IMatterGridCell cell && !cell.isValidMatterCell())) {
			self = self.setValue(MAPPING_CONNECTION_PROP[normal.ordinal()], false);
			updated = true;
		}

		if (updated) {
			level.setBlock(position_self, self, UPDATE_CLIENTS);
		}

		super.neighborChanged(self, level, position_self, block_notifier, position_notifier, unknown_flag);
	}

	// tiles
	/*@Override
	public void onNeighborChange(BlockState state, LevelReader world, BlockPos pos, BlockPos neighbor) {
		super.onNeighborChange(state, world, pos, neighbor);
	}*/

	@Override
	protected void createBlockStateDefinition(StateDefinition.Builder<Block, BlockState> builder) {
		builder.add(CONNECTION_SOUTH,
			CONNECTION_WEST,
			CONNECTION_EAST,
			CONNECTION_NORTH,
			CONNECTION_UP,
			CONNECTION_DOWN);
	}
}
