package ru.dbotthepony.mc.otm.block.entity.worker;

import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.DoubleTag;
import net.minecraft.nbt.StringTag;
import net.minecraft.nbt.Tag;
import net.minecraft.world.item.ItemStack;

import javax.annotation.Nullable;
import java.math.BigDecimal;

public record MachineJob(ItemStack stack, double ticks_processing_time, BigDecimal power_consumption_multiplier,
                         CompoundTag data) {
	public MachineJob(ItemStack stack, double ticks_processing_time, BigDecimal power_consumption_multiplier, CompoundTag data) {
		this.stack = stack;
		this.ticks_processing_time = ticks_processing_time;
		this.power_consumption_multiplier = power_consumption_multiplier;
		this.data = data;
	}

	public MachineJob(ItemStack stack, double ticks_processing_time, BigDecimal power_consumption_multiplier) {
		this(stack, ticks_processing_time, power_consumption_multiplier, new CompoundTag());
	}

	public MachineJob(ItemStack stack, double ticks_processing_time) {
		this(stack, ticks_processing_time, BigDecimal.ONE);
	}

	public CompoundTag serializeNBT() {
		CompoundTag store_job = new CompoundTag();

		store_job.put("data", data);
		store_job.put("stack", stack.serializeNBT());
		store_job.putDouble("ticks_processing_time", ticks_processing_time);
		store_job.putString("power_consumption_multiplier", power_consumption_multiplier.toString());

		return store_job;
	}

	@Nullable
	public static MachineJob deserializeNBT(@Nullable Tag nbt) {
		if (nbt == null)
			return null;

		if (nbt instanceof CompoundTag tag) {
			if (tag.get("stack") instanceof CompoundTag stack_tag && tag.get("ticks_processing_time") instanceof DoubleTag ticks_processing_time && tag.get("power_consumption_multiplier") instanceof StringTag power_consumption_multiplier) {
				ItemStack stack = ItemStack.of(stack_tag);

				if (!stack.isEmpty()) {
					return new MachineJob(stack, ticks_processing_time.getAsDouble(), new BigDecimal(power_consumption_multiplier.getAsString()), tag.getCompound("data"));
				}
			}
		}

		return null;
	}
}
