package ru.dbotthepony.mc.otm.capability.drive;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.util.INBTSerializable;
import ru.dbotthepony.mc.otm.menu.DriveViewerMenu;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import java.util.List;
import java.util.UUID;
import java.util.function.Predicate;

@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public interface IMatteryDrive {
	record StoredStack(ItemStack stack, UUID id) {};

	List<StoredStack> getItems();

	boolean isDirty();
	void markDirty();
	void markClean();

	/**
	 * @param item
	 * @return all items belonging to passed class
	 */
	List<StoredStack> findItems(Item item);

	/**
	 * @param stack
	 * @return all items that match this itemstack
	 */
	List<StoredStack> findItems(ItemStack stack);

	ItemStack getItem(UUID id);

	void addListener(IItemViewListener listener);
	void removeListener(IItemViewListener listener);

	int getStoredCount();
	int getCapacity();

	ItemStack insertItem(ItemStack item, boolean simulate);

	default ItemStack extractItem(UUID id, int amount, boolean simulate) {
		return extractItem(id, amount, true, simulate);
	}

	ItemStack extractItem(UUID id, int amount, boolean obey_stack_size, boolean simulate);

	// not extending INBTSerializable to avoid serializing it as forgecaps
	CompoundTag serializeNBT();
	void deserializeNBT(CompoundTag nbt);
}
