package ru.dbotthepony.mc.otm.item;

import net.minecraft.ChatFormatting;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.item.*;
import net.minecraft.world.level.Level;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.capabilities.ICapabilityProvider;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.energy.CapabilityEnergy;
import net.minecraftforge.fml.loading.FMLEnvironment;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.IMatteryEnergyStorage;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.menu.FormattingHelper;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;

public class ItemBattery extends Item {
	public class BatteryMatteryCapability implements IMatteryEnergyStorage, ICapabilityProvider {
		private final BigDecimal storage;
		private final BigDecimal max_input;
		private final BigDecimal max_output;
		private final boolean is_creative;
		private BigDecimal energy;
		private String _energy;

		private final ItemStack stack;

		private BigDecimal energy() {
			CompoundTag tag = stack.getOrCreateTag();

			if (tag.contains("otm_energy")) {
				String get_energy = tag.getString("otm_energy");

				if (Objects.equals(_energy, get_energy)) {
					return energy;
				}

				energy = new BigDecimal(tag.getString("otm_energy"));
				_energy = get_energy;

				return energy;
			}

			return BigDecimal.ZERO;
		}

		private void energy(BigDecimal value) {
			energy = value;
			stack.getOrCreateTag().putString("otm_energy", value.toString());
		}

		public BatteryMatteryCapability(ItemStack stack, BigDecimal storage, BigDecimal max_input, BigDecimal max_output) {
			this.stack = stack;
			is_creative = false;
			this.storage = storage;
			this.max_input = max_input;
			this.max_output = max_output;
		}

		public BatteryMatteryCapability(ItemStack stack) {
			this.stack = stack;
			is_creative = true;
			storage = MatteryCapability.LONG_MAX_VALUE;
			max_input = MatteryCapability.LONG_MAX_VALUE;
			max_output = MatteryCapability.LONG_MAX_VALUE;
		}

		@Nonnull
		@Override
		public BigDecimal extractEnergyOuter(BigDecimal howMuch, boolean simulate) {
			return extractEnergyInner(howMuch, simulate);
		}

		@Nonnull
		@Override
		public BigDecimal extractEnergyInner(BigDecimal howMuch, boolean simulate) {
			if (is_creative)
				return howMuch;

			BigDecimal new_energy = energy().subtract(howMuch.min(max_output), MatteryCapability.ROUND_RULES).max(BigDecimal.ZERO);
			BigDecimal diff = energy().subtract(new_energy);

			if (!simulate) {
				energy(new_energy);
			}

			return diff;
		}

		@Nonnull
		@Override
		public BigDecimal receiveEnergyOuter(BigDecimal howMuch, boolean simulate) {
			return receiveEnergyInner(howMuch, simulate);
		}

		@Nonnull
		@Override
		public BigDecimal receiveEnergyInner(BigDecimal howMuch, boolean simulate) {
			if (is_creative)
				return howMuch;

			BigDecimal new_energy = energy().add(howMuch.min(max_input), MatteryCapability.ROUND_RULES).min(storage);
			BigDecimal diff = new_energy.subtract(energy());

			if (!simulate) {
				energy(new_energy);
			}

			return diff;
		}

		@Override
		public BigDecimal getMissingPower() {
			if (is_creative)
				return MatteryCapability.LONG_MAX_VALUE;

			return IMatteryEnergyStorage.super.getMissingPower();
		}

		@Nonnull
		@Override
		public BigDecimal getBatteryLevel() {
			if (is_creative)
				return MatteryCapability.LONG_MAX_VALUE;

			return energy();
		}

		@Nonnull
		@Override
		public BigDecimal getMaxBatteryLevel() {
			if (is_creative)
				return MatteryCapability.LONG_MAX_VALUE;

			return storage;
		}

		private final LazyOptional<IMatteryEnergyStorage> resolver = LazyOptional.of(() -> this);

		@Nonnull
		@Override
		public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
			if (cap == MatteryCapability.ENERGY || cap == CapabilityEnergy.ENERGY)
				return resolver.cast();

			return LazyOptional.empty();
		}

		@Override
		public boolean canExtract() {
			return true;
		}

		@Override
		public boolean canReceive() {
			return true;
		}
	}

	private final boolean is_creative;

	public final BigDecimal storage;
	public final BigDecimal receive;
	public final BigDecimal extract;

	private final Component THROUGHPUT;
	private static final Component INFINITE_STORAGE = new TranslatableComponent("otm.item.power.infinite.storage").withStyle(ChatFormatting.GRAY);

	public ItemBattery(BigDecimal storage, BigDecimal receive, BigDecimal extract) {
		super(new Properties().stacksTo(1).tab(OverdriveThatMatters.CREATIVE_TAB));
		is_creative = false;
		this.storage = storage;
		this.receive = receive;
		this.extract = extract;
		THROUGHPUT = new TranslatableComponent("otm.item.power.normal.throughput", FormattingHelper.formatPower(receive), FormattingHelper.formatPower(extract)).withStyle(ChatFormatting.GRAY);
	}

	public ItemBattery() {
		super(new Properties().stacksTo(1).rarity(Rarity.EPIC).tab(OverdriveThatMatters.CREATIVE_TAB));
		is_creative = true;
		this.storage = MatteryCapability.LONG_MAX_VALUE;
		this.receive = MatteryCapability.LONG_MAX_VALUE;
		this.extract = MatteryCapability.LONG_MAX_VALUE;
		THROUGHPUT = new TranslatableComponent("otm.item.power.infinite.throughput").withStyle(ChatFormatting.GRAY);
	}

	@Override
	public void appendHoverText(ItemStack stack, @Nullable Level p_41422_, List<Component> p_41423_, TooltipFlag p_41424_) {
		super.appendHoverText(stack, p_41422_, p_41423_, p_41424_);

		if (is_creative) {
			p_41423_.add(INFINITE_STORAGE);
			p_41423_.add(THROUGHPUT);
		} else {
			stack.getCapability(MatteryCapability.ENERGY).ifPresent(cap ->
				p_41423_.add(new TranslatableComponent("otm.item.power.normal.storage", FormattingHelper.formatPower(cap.getBatteryLevel()), FormattingHelper.formatPower(cap.getMaxBatteryLevel())).withStyle(ChatFormatting.GRAY))
			);

			p_41423_.add(THROUGHPUT);
		}
	}

	@Nullable
	@Override
	public ICapabilityProvider initCapabilities(ItemStack stack, @Nullable CompoundTag nbt) {
		if (is_creative)
			return new BatteryMatteryCapability(stack);

		return new BatteryMatteryCapability(stack, storage, receive, extract);
	}
}
