package ru.dbotthepony.mc.otm.menu;

import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.inventory.ContainerData;
import net.minecraft.world.inventory.MenuType;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.block.entity.BlockEntity;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityMatterBottler;
import ru.dbotthepony.mc.otm.capability.IMatterHandler;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.menu.slot.MatterySlot;
import ru.dbotthepony.mc.otm.menu.widget.MatterLevelWidget;
import ru.dbotthepony.mc.otm.menu.widget.ProgressGaugeWidget;
import ru.dbotthepony.mc.otm.network.MatterBottlerSwitchPacket;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;
import ru.dbotthepony.mc.otm.screen.UVWindingOrder;

import javax.annotation.Nullable;

public class MatterBottlerMenu extends PoweredMatteryMenu {
	public MatterBottlerMenu(int p_38852_, Inventory inventory) {
		this(p_38852_, inventory, null);
	}

	public ContainerData work_flow = new ContainerData() {
		private int value;

		@Override
		public int get(int i) {
			if (tile != null) {
				return ((BlockEntityMatterBottler) tile).getWorkFlow() ? 1 : 0;
			}

			return value;
		}

		@Override
		public void set(int i, int i1) {
			value = i1;
		}

		@Override
		public int getCount() {
			return 1;
		}
	};

	public ProgressGaugeWidget progress;
	public MatterLevelWidget matter_widget;

	public MatterySlot[] work_slots = new MatterySlot[6];

	public MatterBottlerMenu(int p_38852_, Inventory inventory, BlockEntityMatterBottler tile) {
		super(Registry.Menus.MATTER_BOTTLER, p_38852_, inventory, tile);

		var container = tile != null ? tile.work_slots : new SimpleContainer(6);

		if (tile == null) {
			progress = new ProgressGaugeWidget(this) {
				@Override
				public UVWindingOrder getUVWindwing() {
					if (work_flow.get(0) > 0) {
						return UVWindingOrder.U0_V0_U1_V1;
					}

					return UVWindingOrder.U1_V0_U0_V1;
				}
			};

			matter_widget = new MatterLevelWidget(this);
		} else {
			progress = new ProgressGaugeWidget(this, tile::getWorkProgress);
			matter_widget = new MatterLevelWidget(this, tile.matter);
		}

		for (int i = 0; i < container.getContainerSize(); i++) {
			work_slots[i] = new MatterySlot(container, i) {
				@Override
				public boolean mayPlace(ItemStack p_40231_) {
					var cap = p_40231_.getCapability(MatteryCapability.MATTER).resolve();

					if (work_flow.get(0) > 0) {
						return index < 3 && cap.isPresent() && cap.get().getDirection() != IMatterHandler.MatterDirection.EXTRACT;
					}

					return index >= 3 && cap.isPresent() && cap.get().getDirection() != IMatterHandler.MatterDirection.RECEIVE;
				}
			};

			addSlot(work_slots[i]);
		}

		addDataSlots(work_flow);
		addBatterySlot();
		addInventorySlots();
	}

	@Override
	protected int getWorkingSlotStart() {
		return 0;
	}

	@Override
	protected int getWorkingSlotEnd() {
		return 7;
	}

	public void switchBottlerMode() {
		if (tile == null) {
			MatteryNetworking.CHANNEL.sendToServer(new MatterBottlerSwitchPacket());
		} else {
			((BlockEntityMatterBottler) tile).switchWorkFlow();
		}
	}
}
