package ru.dbotthepony.mc.otm.network;

import net.minecraft.resources.ResourceLocation;
import net.minecraft.server.level.ServerPlayer;
import net.minecraftforge.fmllegacy.network.NetworkDirection;
import net.minecraftforge.fmllegacy.network.NetworkRegistry;
import net.minecraftforge.fmllegacy.network.PacketDistributor;
import net.minecraftforge.fmllegacy.network.simple.SimpleChannel;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.item.ItemPortableCondensationDrive;
import ru.dbotthepony.mc.otm.menu.DriveViewerMenu;
import ru.dbotthepony.mc.otm.network.android.*;

import java.util.Optional;

public class MatteryNetworking {
	private static final String PROTOCOL_VERSION = "2";

	private static int next_network_id = 0;

	public static final SimpleChannel CHANNEL = NetworkRegistry.newSimpleChannel(
		new ResourceLocation(OverdriveThatMatters.MOD_ID, "main"),
		() -> PROTOCOL_VERSION,
		PROTOCOL_VERSION::equals,
		PROTOCOL_VERSION::equals
	);

	public static void send(ServerPlayer ply, Object ...messages) {
		var supplier = PacketDistributor.PLAYER.with(() -> ply);

		for (var message : messages) {
			CHANNEL.send(supplier, message);
		}
	}

	public static void send(Object ...messages) {
		for (var message : messages) {
			CHANNEL.sendToServer(message);
		}
	}

	public static void register() {
		CHANNEL.registerMessage(
			next_network_id++,
			AndroidBatteryPacket.class,
			AndroidBatteryPacket::write,
			AndroidBatteryPacket::read,
			AndroidBatteryPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			AndroidEnergyPacket.class,
			AndroidEnergyPacket::write,
			AndroidEnergyPacket::read,
			AndroidEnergyPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			AndroidStatusPacket.class,
			AndroidStatusPacket::write,
			AndroidStatusPacket::read,
			AndroidStatusPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			AndroidFeaturePacket.class,
			AndroidFeaturePacket::write,
			AndroidFeaturePacket::read,
			AndroidFeaturePacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			PatternGridPacket.class,
			PatternGridPacket::write,
			PatternGridPacket::read,
			PatternGridPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			MatterTaskPacket.class,
			MatterTaskPacket::write,
			MatterTaskPacket::read,
			MatterTaskPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			CancelMatterTaskPacket.class,
			CancelMatterTaskPacket::write,
			CancelMatterTaskPacket::read,
			CancelMatterTaskPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			PatternReplicationRequestPacket.class,
			PatternReplicationRequestPacket::write,
			PatternReplicationRequestPacket::read,
			PatternReplicationRequestPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			MatterRegistryPacket.class,
			MatterRegistryPacket::write,
			MatterRegistryPacket::read,
			MatterRegistryPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			AndroidResearchRequestPacket.class,
			AndroidResearchRequestPacket::write,
			AndroidResearchRequestPacket::read,
			AndroidResearchRequestPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			AndroidResearchPacket.class,
			AndroidResearchPacket::write,
			AndroidResearchPacket::read,
			AndroidResearchPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			MatterBottlerSwitchPacket.class,
			MatterBottlerSwitchPacket::write,
			MatterBottlerSwitchPacket::read,
			MatterBottlerSwitchPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			DriveViewerMenu.ClearPacket.class,
			DriveViewerMenu.ClearPacket::write,
			DriveViewerMenu.ClearPacket::read,
			DriveViewerMenu.ClearPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			DriveViewerMenu.StackAddPacket.class,
			DriveViewerMenu.StackAddPacket::write,
			DriveViewerMenu.StackAddPacket::read,
			DriveViewerMenu.StackAddPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			DriveViewerMenu.StackChangePacket.class,
			DriveViewerMenu.StackChangePacket::write,
			DriveViewerMenu.StackChangePacket::read,
			DriveViewerMenu.StackChangePacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			DriveViewerMenu.StackRemovePacket.class,
			DriveViewerMenu.StackRemovePacket::write,
			DriveViewerMenu.StackRemovePacket::read,
			DriveViewerMenu.StackRemovePacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			DriveViewerMenu.InteractPacket.class,
			DriveViewerMenu.InteractPacket::write,
			DriveViewerMenu.InteractPacket::read,
			DriveViewerMenu.InteractPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			SetCarriedPacket.class,
			SetCarriedPacket::write,
			SetCarriedPacket::read,
			SetCarriedPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			DriveViewerMenu.FilterSwitchPacket.class,
			DriveViewerMenu.FilterSwitchPacket::write,
			DriveViewerMenu.FilterSwitchPacket::read,
			DriveViewerMenu.FilterSwitchPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			DriveViewerMenu.FilterSetPacket.class,
			DriveViewerMenu.FilterSetPacket::write,
			DriveViewerMenu.FilterSetPacket::read,
			DriveViewerMenu.FilterSetPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);
	}
}
