package ru.dbotthepony.mc.otm.screen;

import com.mojang.blaze3d.platform.InputConstants;
import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.gui.components.Button;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.inventory.ClickAction;
import net.minecraft.world.inventory.ClickType;
import net.minecraft.world.item.ItemStack;
import ru.dbotthepony.mc.otm.item.ItemPortableCondensationDrive;
import ru.dbotthepony.mc.otm.menu.DriveViewerMenu;
import ru.dbotthepony.mc.otm.menu.slot.MatterySlot;
import ru.dbotthepony.mc.otm.menu.widget.GaugeWidget;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;
import ru.dbotthepony.mc.otm.screen.panels.*;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

public class DriveViewerScreen extends MatteryScreen<DriveViewerMenu> implements MatteryScreen.IMatteryScreenGaugeGetter, MatteryScreen.IMatteryScreenBatteryGetter {
	public static final int FRAME_WIDTH = 210;
	public static final int FRAME_HEIGHT = 110;

	public static final int GRID_WIDTH = 9;
	public static final int GRID_HEIGHT = 5;

	public DriveViewerScreen(DriveViewerMenu menu, Inventory inventory, Component title) {
		super(menu, inventory, title);
	}

	@Override
	public List<GaugeWidget> getGauges() {
		return List.of(menu.battery_widget);
	}

	@Override
	public List<MatterySlot> getBatterySlots() {
		return List.of(menu.battery_slot, menu.drive_slot);
	}

	@Nullable
	@Override
	protected FramePanel makeMainFrame() {
		var frame = new FramePanel(this, null, 0, 0, FRAME_WIDTH, FRAME_HEIGHT, getTitle());

		autoAttachToFrame(frame);

		var view_panels = new ArrayList<EditablePanel>();
		var settings_panels = new ArrayList<EditablePanel>();

		var view_button = frame.addTab(FramePanel.FrameTabPosition.TOP);
		var settings_button = frame.addTab(FramePanel.FrameTabPosition.TOP);

		view_button.bindOnOpen(() -> {
			for (var panel : view_panels) {
				panel.setVisible(true);
			}
		});

		view_button.bindOnClose(() -> {
			for (var panel : view_panels) {
				panel.setVisible(false);
			}
		});

		settings_button.bindOnOpen(() -> {
			for (var panel : settings_panels) {
				panel.setVisible(true);
			}
		});

		settings_button.bindOnClose(() -> {
			for (var panel : settings_panels) {
				panel.setVisible(false);
			}
		});

		var grid = new GridPanel(this, frame, 0, 0, 0, 0, GRID_WIDTH, GRID_HEIGHT);
		grid.setDock(Dock.FILL);
		grid.setDockMargin(2, 2, 2, 2);

		var scroll_bar = new ScrollBarPanel(this, frame, 0, 0, 0);
		scroll_bar.setDock(Dock.RIGHT);
		scroll_bar.setupRowMultiplier(() -> {
			return menu.view.getItems().size() / GRID_WIDTH;
		});

		view_panels.add(grid);
		view_panels.add(scroll_bar);

		for (int i = 0; i < GRID_WIDTH * GRID_HEIGHT; i++) {
			final int index = i;

			new AbstractSlotPanel(this, grid, 0, 0) {
				@Nonnull
				@Override
				protected ItemStack getItemStack() {
					int findex = index + scroll_bar.getScroll(menu.view.getItems().size() / GRID_WIDTH);

					var list = menu.view.getItems();

					if (findex >= list.size()) {
						return ItemStack.EMPTY;
					}

					return list.get(findex).stack();
				}

				@Override
				protected boolean mouseScrolledInner(double mouse_x, double mouse_y, double scroll) {
					return scroll_bar.mouseScrolledInner(mouse_x, mouse_y, scroll);
				}

				@Override
				protected boolean mouseClickedInner(double mouse_x, double mouse_y, int mouse_click_type) {
					int findex = index + scroll_bar.getScroll(GRID_WIDTH);
					var list = menu.view.getItems();
					var action = mouse_click_type == InputConstants.MOUSE_BUTTON_LEFT ? ClickAction.PRIMARY : ClickAction.SECONDARY;

					var type = mouse_click_type == InputConstants.MOUSE_BUTTON_MIDDLE ? ClickType.CLONE : hasShiftDown() ? ClickType.QUICK_MOVE : ClickType.PICKUP;

					MatteryNetworking.send(new DriveViewerMenu.InteractPacket(menu.containerId, findex >= list.size() ? -1 : list.get(findex).id(), type, action));
					return true;
				}
			};
		}

		var dock_left = new FlexGridPanel(this, frame, 0, 0, 90, 0);
		dock_left.setDock(Dock.LEFT);
		dock_left.setDockMargin(4, 0, 4, 0);

		var grid_filter = new FlexGridPanel(this, frame);
		grid_filter.setDock(Dock.FILL);

		settings_panels.add(dock_left);
		settings_panels.add(grid_filter);

		for (int i = 0; i < ItemPortableCondensationDrive.FilterSettings.MAX_FILTERS; i++) {
			final int index = i;

			new AbstractSlotPanel(this, grid_filter, 0, 0) {
				@Nonnull
				@Override
				protected ItemStack getItemStack() {
					var filter = menu.getFilter();

					if (filter == null)
						return ItemStack.EMPTY;

					return filter.items[index];
				}

				private boolean clicking = false;

				@Override
				protected boolean mouseClickedInner(double mouse_x, double mouse_y, int flag) {
					clicking = true;
					return true;
				}

				@Override
				protected boolean mouseReleasedInner(double mouse_x, double mouse_y, int flag) {
					if (clicking)
						MatteryNetworking.send(new DriveViewerMenu.FilterSetPacket(menu.containerId, index, menu.getCarried()));

					clicking = false;

					return true;
				}
			};
		}

		final var no = new TranslatableComponent("otm.filter.no");
		final var yes = new TranslatableComponent("otm.filter.yes");

		var match_nbt = new ButtonPanel(this, dock_left, 0, 0, 90, 20, new TranslatableComponent("otm.filter.match_nbt", no)) {
			@Override
			public void tick() {
				super.tick();

				var filter = menu.getFilter();

				if (filter != null) {
					getOrCreateWidget().setMessage(new TranslatableComponent("otm.filter.match_nbt", filter.match_nbt ? yes : no));
					getOrCreateWidget().active = !isWidgetDisabled();
				} else {
					getOrCreateWidget().active = false;
				}
			}

			@Override
			protected void onPress() {
				super.onPress();

				var filter = menu.getFilter();

				if (filter != null) {
					MatteryNetworking.send(new DriveViewerMenu.FilterSwitchPacket(menu.containerId, DriveViewerMenu.FilterSwitch.MATCH_NBT, !filter.match_nbt));
					disableFor(20);
				}
			}
		};

		var match_tag = new ButtonPanel(this, dock_left, 0, 0, 90, 20, new TranslatableComponent("otm.filter.match_tag", no)) {
			@Override
			public void tick() {
				super.tick();

				var filter = menu.getFilter();

				if (filter != null) {
					getOrCreateWidget().setMessage(new TranslatableComponent("otm.filter.match_tag", filter.match_tag ? yes : no));
					getOrCreateWidget().active = !isWidgetDisabled();
				} else {
					getOrCreateWidget().active = false;
				}
			}

			@Override
			protected void onPress() {
				super.onPress();

				var filter = menu.getFilter();

				if (filter != null) {
					MatteryNetworking.send(new DriveViewerMenu.FilterSwitchPacket(menu.containerId, DriveViewerMenu.FilterSwitch.MATCH_TAG, !filter.match_tag));
					disableFor(20);
				}
			}
		};

		var blacklist = new ButtonPanel(this, dock_left, 0, 0, 90, 20, new TranslatableComponent("otm.filter.blacklist", no)) {
			@Override
			public void tick() {
				super.tick();

				var filter = menu.getFilter();

				if (filter != null) {
					getOrCreateWidget().setMessage(new TranslatableComponent("otm.filter.blacklist", filter.blacklist ? yes : no));
					getOrCreateWidget().active = !isWidgetDisabled();
				} else {
					getOrCreateWidget().active = false;
				}
			}

			@Override
			protected void onPress() {
				super.onPress();

				var filter = menu.getFilter();

				if (filter != null) {
					MatteryNetworking.send(new DriveViewerMenu.FilterSwitchPacket(menu.containerId, DriveViewerMenu.FilterSwitch.BLACKLIST, !filter.blacklist));
					disableFor(20);
				}
			}
		};

		match_nbt.setDockMargin(0, 4, 0, 0);
		match_tag.setDockMargin(0, 4, 0, 0);
		blacklist.setDockMargin(0, 4, 0, 0);

		for (var panel : settings_panels) {
			panel.setVisible(false);
		}

		return frame;
	}
}
